@@ -8,16 +8,15 @@
 
 use crate::input::config;
 use crate::output::comment;
-use crate::output::data_type;
 use crate::output::diagnostic;
 use crate::output::extension;
 use crate::output::path;
 use crate::output::tree;
+use crate::output::type_system::data;
 use std::collections::HashMap;
 use std::collections::HashSet;
 use std::fmt::Debug;
 use std::hash::Hash;
-use std::sync::Arc;
 
 /// Parse/validation context and output node, passed to parser functions along
 /// with a reference to the to-be-parsed input node.
@@ -98,7 +97,7 @@ impl<'a> Context<'a> {
     /// Returns the data type currently associated with the current node. If no
     /// data type was associated yet, this silently returns a reference to an
     /// unresolved type object.
-    pub fn data_type(&self) -> Arc<data_type::DataType> {
+    pub fn data_type(&self) -> data::Type {
         self.output.data_type.clone().unwrap_or_default()
     }
 
@@ -180,7 +179,7 @@ impl<'a> Context<'a> {
     /// Can be called multiple times; only the data type specified for the
     /// final call attached to the node's "return type", but each time a
     /// NodeData::DataType is pushed into the node data as well.
-    pub fn set_data_type(&mut self, data_type: Arc<data_type::DataType>) {
+    pub fn set_data_type(&mut self, data_type: data::Type) {
         if !data_type.is_unresolved() {
             self.push(tree::NodeData::DataType(data_type.clone()));
         }
@@ -192,7 +191,7 @@ impl<'a> Context<'a> {
     /// inputs, but before they start to parse any expressions based on that
     /// schema; after all, the schema defines how (column) references behave.
     /// If the schema isn't known, it may be set to an unresolved type.
-    pub fn set_schema(&mut self, schema: Arc<data_type::DataType>) {
+    pub fn set_schema(&mut self, schema: data::Type) {
         *self
             .state
             .schema_stack
@@ -216,7 +215,7 @@ impl<'a> Context<'a> {
     /// be its parent query, 2 would be its grandparent, etc. Returns Err when
     /// the referenced schema semantically doesn't exist; returns Ok(unresolved
     /// type) when it does but the actual type isn't known.
-    pub fn schema(&self, depth: usize) -> diagnostic::Result<Arc<data_type::DataType>> {
+    pub fn schema(&self, depth: usize) -> diagnostic::Result<data::Type> {
         let len = self.state.schema_stack.len();
         if depth >= len {
             Err(cause!(
@@ -260,8 +259,16 @@ impl<'a> Context<'a> {
         &self.output.data
     }
 
+    /// Returns the URI -> simple extension module map. This includes
+    /// transitive dependencies that don't have an anchor.
+    pub fn extension_modules(
+        &mut self,
+    ) -> &mut HashMap<String, extension::simple::module::Reference> {
+        &mut self.state.extension_modules
+    }
+
     /// Returns the resolver for URI anchors and references.
-    pub fn extension_uris(&mut self) -> &mut Resolver<u32, Arc<extension::YamlInfo>> {
+    pub fn extension_uris(&mut self) -> &mut Resolver<u32, extension::simple::module::Reference> {
         &mut self.state.extension_uris
     }
 
@@ -270,82 +277,80 @@ impl<'a> Context<'a> {
     pub fn define_extension_uri(
         &mut self,
         anchor: u32,
-        uri: Arc<extension::YamlInfo>,
-    ) -> Result<(), (Arc<extension::YamlInfo>, path::PathBuf)> {
+        uri: extension::simple::module::Reference,
+    ) -> Result<(), (extension::simple::module::Reference, path::PathBuf)> {
         self.state
             .extension_uris
             .define(anchor, uri, self.breadcrumb.path.to_path_buf())
     }
 
     /// Returns the resolver for function anchors and references.
-    pub fn fns(&mut self) -> &mut Resolver<u32, Arc<extension::Reference<extension::Function>>> {
+    pub fn functions(
+        &mut self,
+    ) -> &mut Resolver<u32, extension::simple::function::ResolutionResult> {
         &mut self.state.functions
     }
 
     /// Registers a function definition. Shorthand for fns().define(), using
     /// the current path as the registration path.
-    pub fn define_fn(
+    pub fn define_function(
         &mut self,
         anchor: u32,
-        uri: Arc<extension::Reference<extension::Function>>,
-    ) -> Result<
-        (),
-        (
-            Arc<extension::Reference<extension::Function>>,
-            path::PathBuf,
-        ),
-    > {
+        function: extension::simple::function::ResolutionResult,
+    ) -> Result<(), (extension::simple::function::ResolutionResult, path::PathBuf)> {
         self.state
             .functions
-            .define(anchor, uri, self.breadcrumb.path.to_path_buf())
+            .define(anchor, function, self.breadcrumb.path.to_path_buf())
     }
 
     /// Returns the resolver for type anchors and references.
-    pub fn types(&mut self) -> &mut Resolver<u32, Arc<extension::Reference<extension::DataType>>> {
-        &mut self.state.types
+    pub fn type_classes(
+        &mut self,
+    ) -> &mut Resolver<u32, extension::simple::type_class::ResolutionResult> {
+        &mut self.state.type_classes
     }
 
-    /// Registers a type definition. Shorthand for fns().define(), using the
+    /// Registers a type definition. Shorthand for types().define(), using the
     /// current path as the registration path.
     pub fn define_type(
         &mut self,
         anchor: u32,
-        uri: Arc<extension::Reference<extension::DataType>>,
+        class: extension::simple::type_class::ResolutionResult,
     ) -> Result<
         (),
         (
-            Arc<extension::Reference<extension::DataType>>,
+            extension::simple::type_class::ResolutionResult,
             path::PathBuf,
         ),
     > {
         self.state
-            .types
-            .define(anchor, uri, self.breadcrumb.path.to_path_buf())
+            .type_classes
+            .define(anchor, class, self.breadcrumb.path.to_path_buf())
     }
 
     /// Returns the resolver for type variation anchors and references.
-    pub fn tvars(
+    pub fn type_variations(
         &mut self,
-    ) -> &mut Resolver<u32, Arc<extension::Reference<extension::TypeVariation>>> {
+    ) -> &mut Resolver<u32, extension::simple::type_variation::ResolutionResult> {
         &mut self.state.type_variations
     }
 
-    /// Registers a type definition. Shorthand for fns().define(), using the
+    /// Registers a type definition. Shorthand for tvars().define(), using the
     /// current path as the registration path.
-    pub fn define_tvar(
+    pub fn define_type_variation(
         &mut self,
         anchor: u32,
-        uri: Arc<extension::Reference<extension::TypeVariation>>,
+        variation: extension::simple::type_variation::ResolutionResult,
     ) -> Result<
         (),
         (
-            Arc<extension::Reference<extension::TypeVariation>>,
+            extension::simple::type_variation::ResolutionResult,
             path::PathBuf,
         ),
     > {
         self.state
             .type_variations
-            .define(anchor, uri, self.breadcrumb.path.to_path_buf())
+            .define(anchor, variation, self.breadcrumb.path.to_path_buf())
     }
 
     /// Returns the resolver for protobuf Any types present in the
@@ -386,18 +391,6 @@ impl<'a> Context<'a> {
         (allowed, path)
     }
 
-    /// Returns a mutable reference to the Option that possibly contains the
-    /// YAML data object under construction.
-    pub fn yaml_data_opt(&mut self) -> &mut Option<extension::YamlData> {
-        &mut self.state.yaml_data
-    }
-
-    /// Returns a mutable reference to the YAML data object under construction.
-    /// Panics if we're not currently constructing YAML data.
-    pub fn yaml_data(&mut self) -> &mut extension::YamlData {
-        self.state.yaml_data.as_mut().unwrap()
-    }
-
     /// Returns the path leading up to the current node.
     pub fn path(&self) -> &path::Path<'a> {
         &self.breadcrumb.path
@@ -430,13 +423,21 @@ impl<'a> Context<'a> {
     pub fn field_parsed<S: AsRef<str>>(&mut self, field: S) -> bool {
         self.breadcrumb.fields_parsed.contains(field.as_ref())
     }
+
+    /// Provides access to the URI recursion stack. When a URI is about to be
+    /// parsed, it is pushed into the vec; when parsing completes, it is popped
+    /// off the stack. The URI resolution function uses this to check for
+    /// duplicates and recursion depth.
+    pub fn uri_stack(&mut self) -> &mut Vec<String> {
+        &mut self.state.uri_stack
+    }
 }
 
-#[derive(Clone, Debug, Default)]
+#[derive(Clone, Debug)]
 pub struct Resolver<K, V>
 where
-    K: Clone + Debug + Default + Eq + Hash,
-    V: Clone + Debug + Default,
+    K: Eq + Hash + Clone,
+    V: Clone,
 {
     /// Map of keys that have been registered thus far to their value and to
     /// the path from which they were registered.
@@ -447,10 +448,23 @@ where
     used: HashSet<K>,
 }
 
+impl<K, V> Default for Resolver<K, V>
+where
+    K: Eq + Hash + Clone,
+    V: Clone,
+{
+    fn default() -> Self {
+        Self {
+            map: Default::default(),
+            used: Default::default(),
+        }
+    }
+}
+
 impl<K, V> Resolver<K, V>
 where
-    K: Clone + Debug + Default + Eq + Hash,
-    V: Clone + Debug + Default,
+    K: Eq + Hash + Clone,
+    V: Clone,
 {
     /// Creates a new resolver.
     pub fn new() -> Self {
@@ -497,17 +511,21 @@ where
 /// Global state information tracked by the validation logic.
 #[derive(Default)]
 pub struct State {
+    /// List of all resolved simple extension modules by URI, including
+    /// transitive dependencies.
+    pub extension_modules: HashMap<String, extension::simple::module::Reference>,
+
     /// URI anchor resolver.
-    pub extension_uris: Resolver<u32, Arc<extension::YamlInfo>>,
+    pub extension_uris: Resolver<u32, extension::simple::module::Reference>,
 
     /// YAML-defined function anchor resolver.
-    pub functions: Resolver<u32, Arc<extension::Reference<extension::Function>>>,
+    pub functions: Resolver<u32, extension::simple::function::ResolutionResult>,
 
     /// YAML-defined data type anchor resolver.
-    pub types: Resolver<u32, Arc<extension::Reference<extension::DataType>>>,
+    pub type_classes: Resolver<u32, extension::simple::type_class::ResolutionResult>,
 
     /// YAML-defined type variation anchor resolver.
-    pub type_variations: Resolver<u32, Arc<extension::Reference<extension::TypeVariation>>>,
+    pub type_variations: Resolver<u32, extension::simple::type_variation::ResolutionResult>,
 
     /// Protobuf Any type URL resolver.
     pub proto_any_types: Resolver<String, ()>,
@@ -520,10 +538,10 @@ pub struct State {
     /// a relation tree, but no schema is known yet (in terms of dataflow,
     /// we're still in the time before the input relation has created a
     /// stream).
-    pub schema_stack: Vec<Option<Arc<data_type::DataType>>>,
+    pub schema_stack: Vec<Option<data::Type>>,
 
-    /// The YAML data object under construction, if any.
-    pub yaml_data: Option<extension::YamlData>,
+    /// Stack for URIs being parsed. Used to detect recursion and limit depth.
+    pub uri_stack: Vec<String>,
 }
 
 /// Breadcrumbs structure. Each breadcrumb is associated with a node, and
