@@ -3,9 +3,9 @@
 //! Module for parsing/validating literals.
 
 use crate::input::proto::substrait;
-use crate::output::data_type;
 use crate::output::diagnostic;
 use crate::output::extension;
+use crate::output::type_system::data;
 use crate::parse::context;
 use crate::parse::extensions;
 use crate::parse::types;
@@ -64,7 +64,7 @@ pub struct Literal {
 
     /// The data type of the literal. LiteralValue must be a valid instance of
     /// this.
-    data_type: Arc<data_type::DataType>,
+    data_type: data::Type,
 }
 
 /// Converts a value in microseconds since the epoch to a chrono::NaiveDateTime.
@@ -86,7 +86,7 @@ fn to_date_time_str(micros: i64, fmt: &str) -> String {
 
 impl Literal {
     /// Shorthand for a new null literal.
-    pub fn new_null(data_type: Arc<data_type::DataType>) -> Literal {
+    pub fn new_null(data_type: data::Type) -> Literal {
         Literal {
             value: LiteralValue::Null,
             data_type,
@@ -96,33 +96,28 @@ impl Literal {
     /// Shorthand for a new simple literal.
     fn new_simple(
         value: LiteralValue,
-        simple: data_type::Simple,
+        simple: data::class::Simple,
         nullable: bool,
-        variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+        variation: data::Variation,
     ) -> diagnostic::Result<Literal> {
         Ok(Literal {
             value,
-            data_type: data_type::DataType::new(
-                data_type::Class::Simple(simple),
-                nullable,
-                variation,
-                vec![],
-            )?,
+            data_type: data::new_type(data::Class::Simple(simple), nullable, variation, vec![])?,
         })
     }
 
     /// Shorthand for a new compound literal.
-    fn new_compound<T: Into<data_type::Parameter>>(
+    fn new_compound<T: Into<data::Parameter>>(
         value: LiteralValue,
-        compound: data_type::Compound,
+        compound: data::class::Compound,
         nullable: bool,
-        variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+        variation: data::Variation,
         args: Vec<T>,
     ) -> diagnostic::Result<Literal> {
         Ok(Literal {
             value,
-            data_type: data_type::DataType::new(
-                data_type::Class::Compound(compound),
+            data_type: data::new_type(
+                data::Class::Compound(compound),
                 nullable,
                 variation,
                 args.into_iter().map(|x| x.into()).collect(),
@@ -131,7 +126,7 @@ impl Literal {
     }
 
     /// Returns the data type of this literal.
-    pub fn data_type(&self) -> &Arc<data_type::DataType> {
+    pub fn data_type(&self) -> &data::Type {
         &self.data_type
     }
 }
@@ -156,24 +151,24 @@ impl Describe for Literal {
             LiteralValue::Boolean(true) => write!(f, "true"),
             LiteralValue::Boolean(false) => write!(f, "false"),
             LiteralValue::Integer(i) => match self.data_type.class() {
-                data_type::Class::Simple(data_type::Simple::I8) => write!(f, "{i}i8"),
-                data_type::Class::Simple(data_type::Simple::I16) => write!(f, "{i}i16"),
-                data_type::Class::Simple(data_type::Simple::I32) => write!(f, "{i}i32"),
-                data_type::Class::Simple(data_type::Simple::I64) => write!(f, "{i}i64"),
-                data_type::Class::Simple(data_type::Simple::Timestamp) => {
+                data::Class::Simple(data::class::Simple::I8) => write!(f, "{i}i8"),
+                data::Class::Simple(data::class::Simple::I16) => write!(f, "{i}i16"),
+                data::Class::Simple(data::class::Simple::I32) => write!(f, "{i}i32"),
+                data::Class::Simple(data::class::Simple::I64) => write!(f, "{i}i64"),
+                data::Class::Simple(data::class::Simple::Timestamp) => {
                     write!(f, "{}", to_date_time_str(*i, "%Y-%m-%d %H:%M:%S%.6f"))
                 }
-                data_type::Class::Simple(data_type::Simple::TimestampTz) => {
+                data::Class::Simple(data::class::Simple::TimestampTz) => {
                     write!(f, "{} UTC", to_date_time_str(*i, "%Y-%m-%d %H:%M:%S%.6f"))
                 }
-                data_type::Class::Simple(data_type::Simple::Date) => {
+                data::Class::Simple(data::class::Simple::Date) => {
                     write!(
                         f,
                         "{}",
                         to_date_time_str(i.saturating_mul(24 * 60 * 60 * 1_000_000), "%Y-%m-%d")
                     )
                 }
-                data_type::Class::Simple(data_type::Simple::Time) => {
+                data::Class::Simple(data::class::Simple::Time) => {
                     write!(f, "{}", to_date_time_str(*i, "%H:%M:%S%.6f"))
                 }
                 _ => write!(f, "{i}"),
@@ -183,8 +178,8 @@ impl Describe for Literal {
                 write!(f, "{:3.1$}", float_pretty_print::PrettyPrintFloat(*v), max)
             }
             LiteralValue::Data16(d) => match self.data_type.class() {
-                data_type::Class::Compound(data_type::Compound::Decimal) => {
-                    if let Some(scale) = self.data_type.int_parameter(1) {
+                data::Class::Compound(data::class::Compound::Decimal) => {
+                    if let Some(scale) = self.data_type.integer_parameter(1) {
                         if d < &0 {
                             write!(f, "-")?;
                         }
@@ -192,7 +187,7 @@ impl Describe for Literal {
                         let s = 10u128.pow(scale as u32);
                         if self
                             .data_type
-                            .int_parameter(0)
+                            .integer_parameter(0)
                             .map(|precision| scale < precision)
                             .unwrap_or(true)
                         {
@@ -207,7 +202,7 @@ impl Describe for Literal {
                         util::string::describe_binary(f, &d.to_le_bytes(), limit)
                     }
                 }
-                data_type::Class::Simple(data_type::Simple::Uuid) => {
+                data::Class::Simple(data::class::Simple::Uuid) => {
                     let b = d.to_ne_bytes();
                     write!(
                         f,
@@ -220,10 +215,10 @@ impl Describe for Literal {
             LiteralValue::String(s) => util::string::describe_string(f, s, limit),
             LiteralValue::Binary(b) => util::string::describe_binary(f, b, limit),
             LiteralValue::Interval(a, b) => match self.data_type.class() {
-                data_type::Class::Simple(data_type::Simple::IntervalYear) => {
+                data::Class::Simple(data::class::Simple::IntervalYear) => {
                     write!(f, "{a}y{b:+}m")
                 }
-                data_type::Class::Simple(data_type::Simple::IntervalDay) => {
+                data::Class::Simple(data::class::Simple::IntervalDay) => {
                     let s = b / 1000000;
                     let us = if b > &0 { *b } else { -b } % 1000000;
                     write!(f, "{a}d{s:+}.{us:06}s")
@@ -231,7 +226,7 @@ impl Describe for Literal {
                 _ => write!(f, "({a}, {b})"),
             },
             LiteralValue::Items(x) => match self.data_type.class() {
-                data_type::Class::Compound(data_type::Compound::Struct) => {
+                data::Class::Compound(data::class::Compound::Struct) => {
                     write!(f, "(")?;
                     util::string::describe_sequence(f, x, limit, 20, |f, value, index, limit| {
                         write!(f, ".{index}: ")?;
@@ -239,7 +234,7 @@ impl Describe for Literal {
                     })?;
                     write!(f, ")")
                 }
-                data_type::Class::Compound(data_type::Compound::NamedStruct) => {
+                data::Class::Compound(data::class::Compound::NamedStruct) => {
                     write!(f, "(")?;
                     util::string::describe_sequence(f, x, limit, 20, |f, value, index, limit| {
                         if let Some(name) = self
@@ -256,7 +251,7 @@ impl Describe for Literal {
                     })?;
                     write!(f, ")")
                 }
-                data_type::Class::Compound(data_type::Compound::List) => {
+                data::Class::Compound(data::class::Compound::List) => {
                     write!(f, "[")?;
                     util::string::describe_sequence(f, x, limit, 20, |f, value, _, limit| {
                         value.describe(f, limit)
@@ -272,7 +267,7 @@ impl Describe for Literal {
                 }
             },
             LiteralValue::Pairs(x) => match self.data_type.class() {
-                data_type::Class::Compound(data_type::Compound::Map) => {
+                data::Class::Compound(data::class::Compound::Map) => {
                     write!(f, "{{")?;
                     util::string::describe_sequence(
                         f,
@@ -321,139 +316,175 @@ impl std::fmt::Display for Literal {
 /// Parses a boolean literal.
 fn parse_boolean(
     x: &bool,
-    _y: &mut context::Context,
+    y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     Literal::new_simple(
         LiteralValue::Boolean(*x),
-        data_type::Simple::Boolean,
+        data::class::Simple::Boolean,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Simple(data::class::Simple::Boolean),
+        ),
     )
 }
 
 /// Parses an i8 literal.
 fn parse_i8(
     x: &i32,
-    _y: &mut context::Context,
+    y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     let x = i8::try_from(*x)
         .map_err(|_| cause!(ExpressionIllegalLiteralValue, "i8 value out of range"))?;
     Literal::new_simple(
         LiteralValue::Integer(x as i64),
-        data_type::Simple::I8,
+        data::class::Simple::I8,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Simple(data::class::Simple::I8),
+        ),
     )
 }
 
 /// Parses an i16 literal.
 fn parse_i16(
     x: &i32,
-    _y: &mut context::Context,
+    y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     let x = i16::try_from(*x)
         .map_err(|_| cause!(ExpressionIllegalLiteralValue, "i16 value out of range"))?;
     Literal::new_simple(
         LiteralValue::Integer(x as i64),
-        data_type::Simple::I16,
+        data::class::Simple::I16,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Simple(data::class::Simple::I16),
+        ),
     )
 }
 
 /// Parses an i32 literal.
 fn parse_i32(
     x: &i32,
-    _y: &mut context::Context,
+    y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     Literal::new_simple(
         LiteralValue::Integer(*x as i64),
-        data_type::Simple::I32,
+        data::class::Simple::I32,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Simple(data::class::Simple::I32),
+        ),
     )
 }
 
 /// Parses an i64 literal.
 fn parse_i64(
     x: &i64,
-    _y: &mut context::Context,
+    y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     Literal::new_simple(
         LiteralValue::Integer(*x),
-        data_type::Simple::I64,
+        data::class::Simple::I64,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Simple(data::class::Simple::I64),
+        ),
     )
 }
 
 /// Parses an fp32 literal.
 fn parse_fp32(
     x: &f32,
-    _y: &mut context::Context,
+    y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     Literal::new_simple(
         LiteralValue::Float(*x as f64),
-        data_type::Simple::Fp32,
+        data::class::Simple::Fp32,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Simple(data::class::Simple::Fp32),
+        ),
     )
 }
 
 /// Parses an fp64 literal.
 fn parse_fp64(
     x: &f64,
-    _y: &mut context::Context,
+    y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     Literal::new_simple(
         LiteralValue::Float(*x),
-        data_type::Simple::Fp64,
+        data::class::Simple::Fp64,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Simple(data::class::Simple::Fp64),
+        ),
     )
 }
 
 /// Parses a string literal.
 fn parse_string(
     x: &str,
-    _y: &mut context::Context,
+    y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     Literal::new_simple(
         LiteralValue::String(x.to_string()),
-        data_type::Simple::String,
+        data::class::Simple::String,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Simple(data::class::Simple::String),
+        ),
     )
 }
 
 /// Parses a binary literal.
 fn parse_binary(
     x: &[u8],
-    _y: &mut context::Context,
+    y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     Literal::new_simple(
         LiteralValue::Binary(x.to_owned()),
-        data_type::Simple::Binary,
+        data::class::Simple::Binary,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Simple(data::class::Simple::Binary),
+        ),
     )
 }
 
@@ -462,7 +493,7 @@ fn parse_timestamp(
     x: &i64,
     y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     let dt = to_date_time(*x)?;
     if dt < chrono::NaiveDate::from_ymd(1000, 1, 1).and_hms(0, 0, 0)
@@ -477,9 +508,13 @@ fn parse_timestamp(
     }
     Literal::new_simple(
         LiteralValue::Integer(*x),
-        data_type::Simple::Timestamp,
+        data::class::Simple::Timestamp,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Simple(data::class::Simple::Timestamp),
+        ),
     )
 }
 
@@ -488,7 +523,7 @@ fn parse_timestamp_tz(
     x: &i64,
     y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     let dt = to_date_time(*x)?;
     if dt < chrono::NaiveDate::from_ymd(1000, 1, 1).and_hms(0, 0, 0)
@@ -503,9 +538,13 @@ fn parse_timestamp_tz(
     }
     Literal::new_simple(
         LiteralValue::Integer(*x),
-        data_type::Simple::TimestampTz,
+        data::class::Simple::TimestampTz,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Simple(data::class::Simple::TimestampTz),
+        ),
     )
 }
 
@@ -514,7 +553,7 @@ fn parse_date(
     x: &i32,
     y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     let dt = to_date_time((*x as i64).saturating_mul(24 * 60 * 60 * 1_000_000))?;
     if dt < chrono::NaiveDate::from_ymd(1000, 1, 1).and_hms(0, 0, 0)
@@ -529,9 +568,13 @@ fn parse_date(
     }
     Literal::new_simple(
         LiteralValue::Integer(*x as i64),
-        data_type::Simple::Date,
+        data::class::Simple::Date,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Simple(data::class::Simple::Date),
+        ),
     )
 }
 
@@ -540,7 +583,7 @@ fn parse_time(
     x: &i64,
     y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     if *x < 0 || *x >= 24 * 60 * 60 * 1_000_000 {
         diagnostic!(
@@ -552,9 +595,13 @@ fn parse_time(
     }
     Literal::new_simple(
         LiteralValue::Integer(*x),
-        data_type::Simple::Time,
+        data::class::Simple::Time,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Simple(data::class::Simple::Time),
+        ),
     )
 }
 
@@ -563,9 +610,8 @@ fn parse_interval_year_to_month(
     x: &substrait::expression::literal::IntervalYearToMonth,
     y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
-    // FIXME: see FIXME for associated type.
     proto_primitive_field!(x, y, years, |x, _| {
         if *x < -10000 || *x > 10000 {
             Err(cause!(
@@ -587,7 +633,7 @@ fn parse_interval_year_to_month(
         }
     });
     let months = x.months.saturating_add(x.years.saturating_mul(12));
-    if months < -120000 || months > 120000 {
+    if !(-120000..=120000).contains(&months) {
         diagnostic!(
             y,
             Error,
@@ -597,9 +643,13 @@ fn parse_interval_year_to_month(
     }
     Literal::new_simple(
         LiteralValue::Interval(x.years.into(), x.months.into()),
-        data_type::Simple::IntervalYear,
+        data::class::Simple::IntervalYear,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Simple(data::class::Simple::IntervalYear),
+        ),
     )
 }
 
@@ -608,9 +658,8 @@ fn parse_interval_day_to_second(
     x: &substrait::expression::literal::IntervalDayToSecond,
     y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
-    // FIXME: see FIXME for associated type.
     proto_primitive_field!(x, y, days, |x, _| {
         if *x < -3650000 || *x > 3650000 {
             Err(cause!(
@@ -629,25 +678,33 @@ fn parse_interval_day_to_second(
             x.days.into(),
             i64::from(x.seconds) * 1000000 + i64::from(x.microseconds),
         ),
-        data_type::Simple::IntervalDay,
+        data::class::Simple::IntervalDay,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Simple(data::class::Simple::IntervalDay),
+        ),
     )
 }
 
 /// Parses a UUID literal.
 fn parse_uuid(
     x: &[u8],
-    _y: &mut context::Context,
+    y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     if let Ok(x) = x.try_into() {
         Literal::new_simple(
             LiteralValue::Data16(i128::from_ne_bytes(x)),
-            data_type::Simple::Uuid,
+            data::class::Simple::Uuid,
             nullable,
-            variation,
+            extensions::simple::resolve_variation_by_class(
+                y,
+                variations,
+                &data::Class::Simple(data::class::Simple::Uuid),
+            ),
         )
     } else {
         Err(cause!(
@@ -661,15 +718,19 @@ fn parse_uuid(
 /// Parses a fixed-length string literal.
 fn parse_fixed_char(
     x: &str,
-    _y: &mut context::Context,
+    y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     Literal::new_compound(
         LiteralValue::String(x.to_string()),
-        data_type::Compound::FixedChar,
+        data::class::Compound::FixedChar,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Compound(data::class::Compound::FixedChar),
+        ),
         vec![i64::try_from(x.len()).unwrap()],
     )
 }
@@ -679,7 +740,7 @@ fn parse_var_char(
     x: &substrait::expression::literal::VarChar,
     y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     proto_primitive_field!(x, y, length);
     let len: u32 = x.length;
@@ -695,9 +756,13 @@ fn parse_var_char(
     });
     Literal::new_compound(
         LiteralValue::String(x.value.clone()),
-        data_type::Compound::VarChar,
+        data::class::Compound::VarChar,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Compound(data::class::Compound::VarChar),
+        ),
         vec![len as i64],
     )
 }
@@ -705,15 +770,19 @@ fn parse_var_char(
 /// Parses a fixed-length binary literal.
 fn parse_fixed_binary(
     x: &[u8],
-    _y: &mut context::Context,
+    y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     Literal::new_compound(
         LiteralValue::Binary(x.to_owned()),
-        data_type::Compound::FixedBinary,
+        data::class::Compound::FixedBinary,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Compound(data::class::Compound::FixedBinary),
+        ),
         vec![i64::try_from(x.len()).unwrap()],
     )
 }
@@ -723,7 +792,7 @@ fn parse_decimal(
     x: &substrait::expression::literal::Decimal,
     y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     proto_primitive_field!(x, y, precision, |x, _| {
         if *x < 0 {
@@ -756,14 +825,18 @@ fn parse_decimal(
         if val >= range || val <= -range {
             Err(cause!(
                 ExpressionIllegalLiteralValue,
-                "decimal value is out of range for specificied precision and scale"
+                "decimal value is out of range for specified precision and scale"
             ))
         } else {
             Literal::new_compound(
                 LiteralValue::Data16(val),
-                data_type::Compound::Decimal,
+                data::class::Compound::Decimal,
                 nullable,
-                variation,
+                extensions::simple::resolve_variation_by_class(
+                    y,
+                    variations,
+                    &data::Class::Compound(data::class::Compound::Decimal),
+                ),
                 vec![precision, scale],
             )
         }
@@ -777,7 +850,7 @@ fn parse_struct_int(
     x: &substrait::expression::literal::Struct,
     y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     let (values, types): (Vec<_>, Vec<_>) = proto_repeated_field!(x, y, fields, parse_literal)
         .1
@@ -790,9 +863,13 @@ fn parse_struct_int(
         .unzip();
     Literal::new_compound(
         LiteralValue::Items(values),
-        data_type::Compound::Struct,
+        data::class::Compound::Struct,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Compound(data::class::Compound::Struct),
+        ),
         types,
     )
 }
@@ -802,9 +879,9 @@ pub fn parse_struct(
     x: &substrait::expression::literal::Struct,
     y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
-    let literal = parse_struct_int(x, y, nullable, variation)?;
+    let literal = parse_struct_int(x, y, nullable, variations)?;
     y.set_data_type(literal.data_type().clone());
     Ok(literal)
 }
@@ -814,7 +891,7 @@ fn parse_list(
     x: &substrait::expression::literal::List,
     y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     let values: Vec<_> = proto_required_repeated_field!(x, y, values, parse_literal)
         .1
@@ -838,9 +915,13 @@ fn parse_list(
     }
     Literal::new_compound(
         LiteralValue::Items(values),
-        data_type::Compound::List,
+        data::class::Compound::List,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Compound(data::class::Compound::List),
+        ),
         vec![data_type],
     )
 }
@@ -850,7 +931,7 @@ fn parse_map(
     x: &substrait::expression::literal::Map,
     y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     let values: Vec<_> = proto_required_repeated_field!(x, y, key_values, |x, y| {
         let key = proto_required_field!(x, y, key, parse_literal)
@@ -889,9 +970,13 @@ fn parse_map(
     }
     Literal::new_compound(
         LiteralValue::Pairs(values),
-        data_type::Compound::Map,
+        data::class::Compound::Map,
         nullable,
-        variation,
+        extensions::simple::resolve_variation_by_class(
+            y,
+            variations,
+            &data::Class::Compound(data::class::Compound::Map),
+        ),
         vec![key_type, value_type],
     )
 }
@@ -947,7 +1032,7 @@ fn parse_user_defined(
     x: &substrait::expression::literal::UserDefined,
     y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     let extension_type = proto_primitive_field!(
         x,
@@ -957,18 +1042,15 @@ fn parse_user_defined(
     )
     .1;
     proto_required_field!(x, y, value, extensions::advanced::parse_functional_any);
+    let class = if let Some(extension_type) = extension_type {
+        data::Class::UserDefined(extension_type)
+    } else {
+        data::Class::Unresolved
+    };
+    let variation = extensions::simple::resolve_variation_by_class(y, variations, &class);
     Ok(Literal {
         value: LiteralValue::UserDefined,
-        data_type: data_type::DataType::new(
-            if let Some(extension_type) = extension_type {
-                data_type::Class::UserDefined(extension_type)
-            } else {
-                data_type::Class::Unresolved
-            },
-            nullable,
-            variation,
-            vec![],
-        )?,
+        data_type: data::new_type(class, nullable, variation, vec![])?,
     })
 }
 
@@ -977,41 +1059,41 @@ fn parse_literal_type(
     x: &substrait::expression::literal::LiteralType,
     y: &mut context::Context,
     nullable: bool,
-    variation: Option<Arc<extension::Reference<extension::TypeVariation>>>,
+    variations: Option<extension::simple::type_variation::ResolutionResult>,
 ) -> diagnostic::Result<Literal> {
     use substrait::expression::literal::LiteralType;
     match x {
-        LiteralType::Boolean(x) => parse_boolean(x, y, nullable, variation),
-        LiteralType::I8(x) => parse_i8(x, y, nullable, variation),
-        LiteralType::I16(x) => parse_i16(x, y, nullable, variation),
-        LiteralType::I32(x) => parse_i32(x, y, nullable, variation),
-        LiteralType::I64(x) => parse_i64(x, y, nullable, variation),
-        LiteralType::Fp32(x) => parse_fp32(x, y, nullable, variation),
-        LiteralType::Fp64(x) => parse_fp64(x, y, nullable, variation),
-        LiteralType::String(x) => parse_string(x, y, nullable, variation),
-        LiteralType::Binary(x) => parse_binary(x, y, nullable, variation),
-        LiteralType::Timestamp(x) => parse_timestamp(x, y, nullable, variation),
-        LiteralType::TimestampTz(x) => parse_timestamp_tz(x, y, nullable, variation),
-        LiteralType::Date(x) => parse_date(x, y, nullable, variation),
-        LiteralType::Time(x) => parse_time(x, y, nullable, variation),
+        LiteralType::Boolean(x) => parse_boolean(x, y, nullable, variations),
+        LiteralType::I8(x) => parse_i8(x, y, nullable, variations),
+        LiteralType::I16(x) => parse_i16(x, y, nullable, variations),
+        LiteralType::I32(x) => parse_i32(x, y, nullable, variations),
+        LiteralType::I64(x) => parse_i64(x, y, nullable, variations),
+        LiteralType::Fp32(x) => parse_fp32(x, y, nullable, variations),
+        LiteralType::Fp64(x) => parse_fp64(x, y, nullable, variations),
+        LiteralType::String(x) => parse_string(x, y, nullable, variations),
+        LiteralType::Binary(x) => parse_binary(x, y, nullable, variations),
+        LiteralType::Timestamp(x) => parse_timestamp(x, y, nullable, variations),
+        LiteralType::TimestampTz(x) => parse_timestamp_tz(x, y, nullable, variations),
+        LiteralType::Date(x) => parse_date(x, y, nullable, variations),
+        LiteralType::Time(x) => parse_time(x, y, nullable, variations),
         LiteralType::IntervalYearToMonth(x) => {
-            parse_interval_year_to_month(x, y, nullable, variation)
+            parse_interval_year_to_month(x, y, nullable, variations)
         }
         LiteralType::IntervalDayToSecond(x) => {
-            parse_interval_day_to_second(x, y, nullable, variation)
+            parse_interval_day_to_second(x, y, nullable, variations)
         }
-        LiteralType::Uuid(x) => parse_uuid(x, y, nullable, variation),
-        LiteralType::FixedChar(x) => parse_fixed_char(x, y, nullable, variation),
-        LiteralType::VarChar(x) => parse_var_char(x, y, nullable, variation),
-        LiteralType::FixedBinary(x) => parse_fixed_binary(x, y, nullable, variation),
-        LiteralType::Decimal(x) => parse_decimal(x, y, nullable, variation),
-        LiteralType::Struct(x) => parse_struct_int(x, y, nullable, variation),
-        LiteralType::List(x) => parse_list(x, y, nullable, variation),
-        LiteralType::Map(x) => parse_map(x, y, nullable, variation),
+        LiteralType::Uuid(x) => parse_uuid(x, y, nullable, variations),
+        LiteralType::FixedChar(x) => parse_fixed_char(x, y, nullable, variations),
+        LiteralType::VarChar(x) => parse_var_char(x, y, nullable, variations),
+        LiteralType::FixedBinary(x) => parse_fixed_binary(x, y, nullable, variations),
+        LiteralType::Decimal(x) => parse_decimal(x, y, nullable, variations),
+        LiteralType::Struct(x) => parse_struct_int(x, y, nullable, variations),
+        LiteralType::List(x) => parse_list(x, y, nullable, variations),
+        LiteralType::Map(x) => parse_map(x, y, nullable, variations),
         LiteralType::EmptyList(x) => parse_empty_list(x, y),
         LiteralType::EmptyMap(x) => parse_empty_map(x, y),
         LiteralType::Null(x) => parse_null(x, y),
-        LiteralType::UserDefined(x) => parse_user_defined(x, y, nullable, variation),
+        LiteralType::UserDefined(x) => parse_user_defined(x, y, nullable, variations),
     }
 }
 
@@ -1062,15 +1144,15 @@ pub fn parse_literal(
     }
 
     // Parse variation.
-    let variation = if attributes_here {
+    let variations = if attributes_here {
         proto_primitive_field!(
             x,
             y,
             type_variation_reference,
-            extensions::simple::parse_type_variation_reference
+            extensions::simple::parse_type_variation_reference_without_class
         )
         .1
-        .flatten()
+        .unwrap_or_default()
     } else {
         proto_primitive_field!(x, y, type_variation_reference, |x, y| {
             // Send diagnostic only when x is not set to its default value,
@@ -1100,7 +1182,7 @@ pub fn parse_literal(
         literal_type,
         parse_literal_type,
         x.nullable,
-        variation
+        variations
     )
     .1
     .unwrap_or_default();
