@@ -14,10 +14,14 @@ from .substrait_validator import (
     ResultHandle,
     Config as _Config,
     get_diagnostic_codes,
+    get_version as _get_version,
     get_substrait_version as _get_substrait_version,
 )
 from .substrait.plan_pb2 import Plan
-from .substrait.validator.validator_pb2 import ParseResult, Diagnostic, Path
+from .substrait.validator.tree_pb2 import ParseResult, Diagnostic, Path
+
+
+__VERSION__ = _get_version()
 
 
 _JDOT_MACROS = """@macros
@@ -289,6 +293,11 @@ def path_to_string(path: Path) -> str:
     return "".join(elements)
 
 
+def version() -> str:
+    """Returns the version of the validator."""
+    return __VERSION__
+
+
 def substrait_version() -> str:
     """Returns the version of Substrait that the validator was built
     against."""
@@ -396,16 +405,36 @@ def substrait_version() -> str:
         "application of any --override-uri options)."
     ),
 )
+@click.option(
+    "--uri-depth",
+    type=int,
+    default=None,
+    help=(
+        "Sets the maximum recursion depth for resolving transitive "
+        "dependencies. You can specify a negative number to disable "
+        "the limit, or set this to zero to disable URI resolution "
+        "entirely."
+    ),
+)
 @click.option(
     "--help-diagnostics",
     is_flag=True,
     help=("Show a list of all known diagnostic codes and exit."),
 )
+@click.option(
+    "--version",
+    "print_version",
+    is_flag=True,
+    help=("Print the version of the validator and exit."),
+)
 @click.option(
     "--substrait-version",
     "print_substrait_version",
     is_flag=True,
-    help=("Print the version(s) of Substrait that the validator " "supports and exit."),
+    help=(
+        "Print the version of Substrait that the validator was "
+        "built against and exit."
+    ),
 )
 def cli(  # noqa: C901
     infile,
@@ -419,7 +448,9 @@ def cli(  # noqa: C901
     diagnostic_level,
     override_uri,
     use_urllib,
+    uri_depth,
     help_diagnostics,
+    print_version,
     print_substrait_version,
 ):
     """Validate or convert the substrait.Plan represented by INFILE (or stdin
@@ -603,6 +634,11 @@ def cli(  # noqa: C901
         click.echo(substrait_version())
         sys.exit(0)
 
+    # Print validator version if requested.
+    if print_version:
+        click.echo(version())
+        sys.exit(0)
+
     # Parse verbosity level.
     verbosity_level = level_str_to_int(verbosity)
 
@@ -734,6 +770,11 @@ def cli(  # noqa: C901
             fatal(e)
     if use_urllib:
         config.add_urllib_resolver()
+    if uri_depth is not None:
+        if uri_depth < 0:
+            config.set_max_uri_resolution_depth(None)
+        else:
+            config.set_max_uri_resolution_depth(uri_depth)
 
     # Run the parser/validator.
     result = plan_to_result_handle(in_plan, config)
